@@ -1,11 +1,15 @@
 import sys
 from pkg_resources import get_distribution, DistributionNotFound
+import atexit
 
 from . import psmultiprocessing
 from .utils import flatten_results, initialize_kwargs, resolve_kwargs
 from . import pslogger
 
 
+our_backend = None
+
+
 try:
     __version__ = get_distribution(__name__).version
 except DistributionNotFound:  # pragma: no cover
@@ -21,6 +25,7 @@ def lazy_load_ray():
         'init': psray.init,
         'map': psray.map,
         'current_core_count': psray.current_core_count,
+        'current_resources': psray.current_resources,
         'finalize': psray.finalize,
     }
 
@@ -43,6 +48,7 @@ backends = {
         'init': psmultiprocessing.init,
         'map': psmultiprocessing.map,
         'current_core_count': psmultiprocessing.current_core_count,
+        'current_resources': psmultiprocessing.current_resources,
         'finalize': psmultiprocessing.finalize,
     },
     'ray': {
@@ -64,6 +70,9 @@ global_kwargs = {
     'ncores': {'env': 'PARAMSURVEY_NCORES', 'default': None},
     'max_tasks_per_child': {'env': 'PARAMSURVEY_MAX_TASKS_PER_CHILD', 'default': None},
     'vstats': {'env': 'PARAMSURVEY_VSTATS', 'default': 1},
+    'carbon_server': {'default': None, 'type': str},
+    'carbon_port': {'default': None},
+    'carbon_prefix': {'default': None, 'type': str},
 }
 
 
@@ -115,14 +124,25 @@ def init(**kwargs):
     kwargs.pop('pslogger_fd', None)
 
     global our_backend
+
     if backend in backends:
         pslogger.log('initializing paramsurvey {} backend'.format(backend), stderr=verbose)
-        our_backend = backends[backend]
-        our_backend['name'] = backend
-        if 'lazy' in our_backend:
-            our_backend.update(our_backend['lazy']())
+        if our_backend is None:
+            our_backend = backends[backend]
+            our_backend['name'] = backend
+            atexit.register(finalize)
+            if 'lazy' in our_backend:
+                our_backend.update(our_backend['lazy']())
+        else:
+            if our_backend != backends[backend]:
+                raise RuntimeError('paramsurvey.init called multiple times with different backends')
+            else:
+                pslogger.log('paramsurvey warning: init called multiple times, args ignored', stderr=verbose)
+                return
+
         system_kwargs, backend_kwargs, other_kwargs = resolve_kwargs(global_kwargs, kwargs, backend, backends)
         our_backend['init'](system_kwargs, backend_kwargs, **other_kwargs)
+
     else:  # pragma: no cover
         raise ValueError('unknown backend '+backend+', valid backends: '+', '.join(backends.keys()))
 
@@ -138,8 +158,9 @@ def backend():
 
 
 def finalize(*args, **kwargs):
-    '''Finalizes the paramsurvey run. Optional. Useful for situations
-    like doing test coverage analysis with the multiprocessing module.
+    '''Finalizes the paramsurvey run. Needed for doing test coverage
+    analysis with the multiprocessing module. Also prints a reminder
+    of the hidden logfile at exit.
     '''
     return our_backend['finalize'](*args, **kwargs)
 
@@ -154,6 +175,16 @@ def current_core_count(*args, **kwargs):
     return our_backend['current_core_count'](*args, **kwargs)
 
 
+def current_resources(*args, **kwargs):
+    '''Returns a description of the current cluster.
+
+    Returns
+    -------
+    list of resources dictionaries, with keys: num_cores, num_gpus, memory (bytes)
+    '''
+    return our_backend['current_resources'](*args, **kwargs)
+
+
 def map(*args, **kwargs):
     '''Runs a worker function over a list of parameters, returning the results.
 
