@@ -5,12 +5,16 @@ import functools
 import time
 import multiprocessing
 
+import psutil
+
 from . import utils
 from . import stats
 from . import pslogger
 
 pool = None
 our_ncores = None
+finalize_needed = False
+max_tasks_per_child_seen = False
 
 
 def init(system_kwargs, backend_kwargs):
@@ -31,27 +35,54 @@ def init(system_kwargs, backend_kwargs):
     max_tasks_per_child = system_kwargs.pop('max_tasks_per_child', None)
     if max_tasks_per_child:
         backend_kwargs['maxtasksperchild'] = max_tasks_per_child
+        global max_tasks_per_child_seen
+        max_tasks_per_child_seen = True
 
     verbose = system_kwargs['verbose']
     pslogger.log('initializing multiprocessing pool with {} processes'.format(ncores), stderr=verbose)
     pslogger.log('Pool() kwargs are', backend_kwargs, stderr=verbose > 1)
 
     pool = multiprocessing.Pool(**backend_kwargs)
+    global finalize_needed
+    finalize_needed = True
 
 
 def finalize():
-    # needed to make things like pytest coverage reporting work
+    global finalize_needed
+    if not finalize_needed:
+        return
+    finalize_needed = False
+
     pslogger.finalize()
+
     pool.close()
-    pool.join()
+
+    # pytest expects that both pool.close and pool.join are called to collect coverage
+    # pool.join makes sure that the children have all exited
+
+    # pool.join tends to hang when used with max_tasks_per_child
+    # https://bugs.python.org/issue10332
+    # https://bugs.python.org/issue38799
+    # and many more.
+
+    # pool.join hangs if nested Pool __init__ has happened
+    # can't find a bugs.python.org ticket
+
+    if not max_tasks_per_child_seen:
+        pool.join()
 
 
 def _core_count():
     try:
+        # recent Linux
         return len(os.sched_getaffinity(0))
     except (AttributeError, NotImplementedError, OSError):
-        return multiprocessing.cpu_count()
-    # Windows: len(psutil.Process().cpu_affinity())
+        try:
+            # Windows, MacOS, FreeBSD
+            return len(psutil.Process().cpu_affinity())
+        except (AttributeError, NotImplementedError, OSError):
+            # older Linux, MacOS. Can raise NotImplementedError
+            return multiprocessing.cpu_count()
 
 
 def current_core_count():
@@ -61,6 +92,15 @@ def current_core_count():
         return _core_count()
 
 
+def current_resources():
+    # return a Ray-esque resource dict for our one node
+    # in theory /sys/fs/cgroup/memory/ has a useful number, but sadly, it doesn't
+    # XXX document find a gpu:
+    # import torch
+    # use_cuda = torch.cuda_is_available()
+    return [{'num_cores': current_core_count(), 'memory': psutil.virtual_memory().total}]
+
+
 def pick_chunksize(length, cores, factor=4):
     # chunksize computation similar to what Python does for a multiprocessing.Pool
     # except the fudge factor can be changed. bigger factor == smaller chunks.
