@@ -11,19 +11,21 @@ import traceback
 import warnings
 import platform
 import subprocess
-import pickle5
+#import pickle5
 
 import pandas as pd
 from pandas_appender import DF_Appender
 
 from . import stats
 from . import pslogger
+from . import carbon
 
 
 class MapProgress(object):
     '''Class to track progress of a map()'''
     # would be perfect as a dataclass, once python 3.7 is our minimum
-    def __init__(self, name, d={}, verbose=1, progress_dt=None):
+    def __init__(self, name, d={}, verbose=1, progress_dt=None,
+                 carbon_server=None, carbon_port=None, carbon_prefix=None):
         self.name = name
         self.print_last = time.time()
         if progress_dt is not None:
@@ -32,6 +34,14 @@ class MapProgress(object):
             self.print_dt = self.pick_dt(verbose)
         self.print_log_last = time.time()
         self.print_log_dt = 30  # hardwired
+        self.carbon_server = carbon_server
+        self.carbon_port = carbon_port
+        if carbon_prefix:
+            self.carbon_prefix = carbon_prefix.rstrip('.')
+        else:
+            self.carbon_prefix = None
+        if carbon_server and carbon_port:
+            carbon.init()
 
         self.total = d.get('total', 0)
         self.active = d.get('active', 0)
@@ -60,6 +70,7 @@ class MapProgress(object):
 
         # fixup wrapper failures
         if actual_failures > failures:
+            # this seems to happen occasionally in the azure pipeline, disappears on a rerun. always on 38verbose
             pslogger.log('correcting failure count from {} to {}'.format(failures, actual_failures))
             self.failures = actual_failures
         elif actual_failures < failures:
@@ -80,6 +91,17 @@ class MapProgress(object):
 
         if final or last or log_last:
             pslogger.log(self.name, 'progress:', str(self), stderr=final or last)
+            if self.carbon_server and self.carbon_port:
+                carbon.carbon_push_pickle(self.construct_tuples(),
+                                          carbon_server=self.carbon_server,
+                                          carbon_port=self.carbon_port)
+
+    def construct_tuples(self):
+        # [(path, (timestamp, value)), ...]
+        timestamp = time.time()
+
+        attrs = ('total', 'active', 'finished', 'failures', 'exceptions')
+        return [(self.carbon_prefix+'.'+k, (timestamp, getattr(self, k))) for k in attrs]
 
 
 class MapResults(pd.DataFrame):
@@ -215,7 +237,12 @@ def map_prep(psets, name, system_kwargs, chdir, out_subdirs, keep_results=True,
     if system_kwargs['limit'] is not None and system_kwargs['limit'] >= 0:
         psets = psets.iloc[:system_kwargs['limit']]
 
-    system_kwargs['progress'] = MapProgress(name, {'total': len(psets)}, verbose=verbose, progress_dt=progress_dt)
+    carbon_server = system_kwargs['carbon_server']
+    carbon_port = system_kwargs['carbon_port']
+    carbon_prefix = system_kwargs['carbon_prefix']
+    system_kwargs['progress'] = MapProgress(name, {'total': len(psets)},
+                                            verbose=verbose, progress_dt=progress_dt,
+                                            carbon_server=carbon_server, carbon_port=carbon_port, carbon_prefix=carbon_prefix)
 
     if keep_results:
         system_kwargs['results'] = DF_Appender(ignore_index=True)
@@ -298,6 +325,27 @@ def make_pset_ids(psets):
     return ret, pset_ids
 
 
+def check_and_exit_if_nested_pool(user_ret):
+    # File ".../lib/python3.8/multiprocessing/process.py", line 118, in start
+    # assert not _current_process._config.get('daemon'), \
+    # AssertionError: daemonic processes are not allowed to have children
+
+    if 'exception' in user_ret:
+        e = user_ret['exception']
+        tb = user_ret.get('traceback')
+        if 'AssertionError' in e and 'daemonic processes are not allowed to have children' in e:
+            # this is multiprocessing only
+            # after this exception, pool.join() will hang.
+            # if we exit without calling pool.join(), in _run_finalizers this will happen:
+            # AssertionError: Cannot have cache with result_hander not alive
+
+            if tb:
+                pslogger.log(tb, stderr=True)
+            pslogger.log('Your code created its own multiprocessing pool, nesting them causes an exception', stderr=True)
+            sys.stderr.flush()
+            os._exit(1)
+
+
 def handle_return_common(out_func, ret, system_stats, system_kwargs, user_kwargs):
     progress = system_kwargs['progress']
     verbose = system_kwargs['verbose']
@@ -316,6 +364,8 @@ def handle_return_common(out_func, ret, system_stats, system_kwargs, user_kwargs
         pset_id = user_ret['pset']['_pset_id']
 
         if 'exception' in user_ret:
+            check_and_exit_if_nested_pool(user_ret)
+
             progress.failures += 1
             progress.exceptions += 1
             progress.active -= 1
@@ -327,6 +377,7 @@ def handle_return_common(out_func, ret, system_stats, system_kwargs, user_kwargs
             if 'traceback' in user_ret:
                 system_kwargs['pset_ids'][pset_id]['_traceback'] = user_ret['traceback']
                 pslogger.log('traceback:\n' + user_ret['traceback'], stderr=verbose > 1)
+                pslogger.traceback(True)
                 if verbose > 0:
                     print('(traceback is in', pslogger.logger_filename+')', file=sys.stderr)
         else:
@@ -353,6 +404,7 @@ def handle_return_common(out_func, ret, system_stats, system_kwargs, user_kwargs
                 progress.failures += 1
                 pslogger.log('saw exception in a user-supplied out_func:', repr(e), stderr=verbose)
                 pslogger.log('traceback:\n' + traceback.format_exc(), stderr=verbose)
+                pslogger.traceback(True)
 
     progress.report()
     system_stats.report()
@@ -361,7 +413,7 @@ def handle_return_common(out_func, ret, system_stats, system_kwargs, user_kwargs
 def initialize_kwargs(global_kwargs, kwargs):
     for k, v in global_kwargs.items():
         value = None
-        if v['env'] in os.environ:
+        if 'env' in v and v['env'] in os.environ:
             value = os.environ[v['env']]
             v['strong'] = True
         elif kwargs.get(k) is not None:
@@ -459,7 +511,9 @@ def subprocess_run_worker(pset, system_kwargs, user_kwargs):
 
 def pick_factor(args):
     try:
-        size = len(pickle5.dumps(args))
+        # pickle5 causes ray to fail https://github.com/ray-project/ray/issues/19938
+        #size = len(pickle5.dumps(args))
+        size = 0
     except Exception as e:
         if str(e) != "cannot pickle '_io.FileIO' object":  # a test causes this one
             pslogger.log('pick_factor: could not estimate arguments size due to pickle5 exception '+str(e), stderr=True)
