@@ -1,14 +1,16 @@
+#!/usr/bin/env python
+
 import paramsurvey
 import paramsurvey.params
 
 
-def first_stage(pset, system_kwargs, user_kwargs, raw_stats):
+def first_stage(pset, system_kwargs, user_kwargs):
     # compute something expensive related to 'a' and 'b'
     expensive = pset['a'] + pset['b']
     return {'expensive': expensive}
 
 
-def second_stage(pset, system_kwargs, user_kwargs, raw_stats):
+def second_stage(pset, system_kwargs, user_kwargs):
     # compute something using the precomputed 'expensive' and 'c'
     final = pset['expensive'] + pset['c']
     return {'final': final}
@@ -25,7 +27,7 @@ def main():
 
     results = paramsurvey.map(second_stage, psets, verbose=2)
 
-    print(results.df)
+    print(results.to_df())
 
 
 # for Windows, you must have a __name__ == __main__' guard on all executable code in the main program (multiprocessing)
