@@ -21,12 +21,6 @@ pslogger_fd = StringIO()
 def paramsurvey_init(request):
     paramsurvey.init(pslogger_fd=pslogger_fd)
 
-    def finalize():
-        # needed to get pytest multiprocessing coverage
-        paramsurvey.finalize()
-
-    request.session.addfinalizer(finalize)
-
 
 def readouterr_and_dump(capsys):
     captured = capsys.readouterr()
@@ -307,14 +301,17 @@ def test_map_ncores(paramsurvey_init):
         paramsurvey.map(sleep_worker, [{'foo': 1}], ncores=3)
 
 
-def test_invalid_kwarg(paramsurvey_init):
+@pytest.mark.skip(reason='cannot call paramsurvey.init twice in a session')
+def test_invalid_kwarg_init():
     with pytest.raises(TypeError):
         paramsurvey.init(doesnotexist=True)
+
+
+def test_invalid_kwarg(paramsurvey_init):
     with pytest.raises(TypeError):
         paramsurvey.map(sleep_worker, [{}], doesnotexist=True)
     with pytest.raises(TypeError):
         paramsurvey.map(sleep_worker, [{}], ray={'doesnotexist': True}, multiprocessing={'doesnotexist': True})
-    pass
 
 
 def test_overlarge_pset():
@@ -364,3 +361,38 @@ def test_subprocess_run():
         assert isinstance(r.cli, subprocess.CompletedProcess)
         assert r.cli.stdout.rstrip() == '/'
         assert r.cli.returncode == 0
+
+
+def test_resources():
+    resources = paramsurvey.current_resources()
+
+    # should be a list of dicts with known keys
+    for r in resources:
+        print(r)
+        for field in ('num_cores', 'memory'):
+            assert field in r
+            print('r field', r[field])
+            assert float(r[field])
+        assert len(r) <= 3
+
+
+def worker_pset_backend_args(pset, system_kwargs, user_kwargs):
+    print('system_kwargs', system_kwargs)
+    if 'ray' in system_kwargs:
+        return {'r': 'PASS'}
+    return {'r': 'FAIL'}
+
+
+def test_pset_backend_args():
+    print('backend', paramsurvey.backend())
+    if paramsurvey.backend() != 'ray':
+        pytest.skip('this test only valid for ray backend')
+
+    psets = [{'foo': 1, 'ray': {'memory': 100 * 1024 * 1024}}]  # 50 megabyte minimum
+    results = paramsurvey.map(worker_pset_backend_args, psets, name='test_pset_backend_args')
+    assert results.progress.finished == 1
+    assert next(results.itertuples()).r == 'PASS', 'worker saw ray backend pset config'
+
+    psets = [{'foo': 1, 'ray': {'thisoptiondoesnotexist': 1}}]
+    with pytest.raises(TypeError):
+        results = paramsurvey.map(worker_pset_backend_args, psets, name='test_pset_backend_args')
